@@ -13,10 +13,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import threading
+
+import cachetools
 from gnocchiclient import client
 from gnocchiclient import exceptions
-from keystoneauth1 import exceptions as ka_exceptions
-from oslo_config import cfg
 from oslo_serialization import jsonutils
 import pecan
 import wsme
@@ -27,14 +28,6 @@ from aodh.api.controllers.v2 import utils as v2_utils
 from aodh import keystone_client
 
 
-GNOCCHI_OPTS = [
-    cfg.StrOpt('gnocchi_external_project_owner',
-               default="service",
-               help='Project name of resources creator in Gnocchi. '
-               '(For example the Ceilometer project name'),
-]
-
-
 class GnocchiUnavailable(Exception):
     code = 503
 
@@ -57,6 +50,9 @@ class AlarmGnocchiThresholdRule(base.AlarmRule):
     granularity = wsme.wsattr(wtypes.IntegerType(minimum=1), default=60)
     "The time range in seconds over which query"
 
+    cache = cachetools.TTLCache(maxsize=1, ttl=3600)
+    lock = threading.RLock()
+
     @classmethod
     def validate_alarm(cls, alarm):
         alarm_rule = getattr(alarm, "%s_rule" % alarm.type)
@@ -66,10 +62,8 @@ class AlarmGnocchiThresholdRule(base.AlarmRule):
                 'aggregation_method should be in %s not %s' % (
                     cls._get_aggregation_methods(), aggregation_method))
 
-    # NOTE(sileht): once cachetools is in the requirements
-    # enable it
-    # @cachetools.ttl_cache(maxsize=1, ttl=600)
     @staticmethod
+    @cachetools.cached(cache, lock=lock)
     def _get_aggregation_methods():
         conf = pecan.request.cfg
         gnocchi_client = client.Client(
@@ -148,22 +142,6 @@ class AggregationMetricByResourcesLookupRule(AlarmGnocchiThresholdRule):
                                        'resource_type'])
         return rule
 
-    # NOTE(sileht): once cachetools is in the requirements
-    # enable it
-    # cache = cachetools.TTLCache(maxsize=1, ttl=3600)
-    # lock = threading.RLock()
-
-    # @cachetools.cached(cache, lock=lock)
-    @staticmethod
-    def get_external_project_owner():
-        kc = keystone_client.get_client(pecan.request.cfg)
-        project_name = pecan.request.cfg.api.gnocchi_external_project_owner
-        try:
-            project = kc.projects.find(name=project_name)
-            return project.id
-        except ka_exceptions.NotFound:
-            return None
-
     @classmethod
     def validate_alarm(cls, alarm):
         super(AggregationMetricByResourcesLookupRule,
@@ -177,27 +155,14 @@ class AggregationMetricByResourcesLookupRule(AlarmGnocchiThresholdRule):
         except ValueError:
             raise wsme.exc.InvalidInput('rule/query', rule.query)
 
-        conf = pecan.request.cfg
-
         # Scope the alarm to the project id if needed
         auth_project = v2_utils.get_auth_project(alarm.project_id)
         if auth_project:
-
-            perms_filter = {"=": {"created_by_project_id": auth_project}}
-
-            external_project_owner = cls.get_external_project_owner()
-            if external_project_owner:
-                perms_filter = {"or": [
-                    perms_filter,
-                    {"and": [
-                        {"=": {"created_by_project_id":
-                               external_project_owner}},
-                        {"=": {"project_id": auth_project}}]}
-                ]}
-
-            query = {"and": [perms_filter, query]}
+            query = {"and": [{"=": {"created_by_project_id": auth_project}},
+                             query]}
             rule.query = jsonutils.dumps(query)
 
+        conf = pecan.request.cfg
         gnocchi_client = client.Client(
             '1', keystone_client.get_session(conf),
             interface=conf.service_credentials.interface,
