@@ -21,13 +21,13 @@
 import datetime
 import itertools
 import json
-import uuid
 
 import croniter
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import netutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import pecan
 from pecan import rest
 import pytz
@@ -384,7 +384,17 @@ class Alarm(base.Base):
 
     @staticmethod
     def _is_trust_url(url):
-        return url.scheme in ('trust+http', 'trust+https')
+        return url.scheme.startswith('trust+')
+
+    def _get_existing_trust_ids(self):
+        for action in itertools.chain(self.ok_actions or [],
+                                      self.alarm_actions or [],
+                                      self.insufficient_data_actions or []):
+            url = netutils.urlsplit(action)
+            if self._is_trust_url(url):
+                trust_id = url.username
+                if trust_id and url.password == 'delete':
+                    yield trust_id
 
     def update_actions(self, old_alarm=None):
         trustor_user_id = pecan.request.headers.get('X-User-Id')
@@ -395,49 +405,48 @@ class Alarm(base.Base):
         else:
             roles = []
         auth_plugin = pecan.request.environ.get('keystone.token_auth')
+
+        if old_alarm:
+            prev_trust_ids = set(old_alarm._get_existing_trust_ids())
+        else:
+            prev_trust_ids = set()
+        trust_id = prev_trust_ids.pop() if prev_trust_ids else None
+        trust_id_used = False
+
         for actions in (self.ok_actions, self.alarm_actions,
                         self.insufficient_data_actions):
             if actions is not None:
                 for index, action in enumerate(actions[:]):
                     url = netutils.urlsplit(action)
                     if self._is_trust_url(url):
-                        if '@' not in url.netloc:
+                        if '@' in url.netloc:
+                            continue
+                        if trust_id is None:
                             # We have a trust action without a trust ID,
                             # create it
                             trust_id = keystone_client.create_trust_id(
                                 pecan.request.cfg,
                                 trustor_user_id, trustor_project_id, roles,
                                 auth_plugin)
-                            netloc = '%s:delete@%s' % (trust_id, url.netloc)
-                            url = list(url)
-                            url[1] = netloc
-                            actions[index] = urlparse.urlunsplit(url)
+                        if trust_id_used:
+                            pw = ''
                         else:
-                            errmsg = _("trust URL cannot contain a trust ID.")
-                            raise base.ClientSideError(errmsg)
-        if old_alarm:
-            new_actions = list(itertools.chain(
-                self.ok_actions or [],
-                self.alarm_actions or [],
-                self.insufficient_data_actions or []))
-            for action in itertools.chain(
-                    old_alarm.ok_actions or [],
-                    old_alarm.alarm_actions or [],
-                    old_alarm.insufficient_data_actions or []):
-                if action not in new_actions:
-                    self.delete_trust(action)
+                            pw = ':delete'
+                            trust_id_used = True
+                        netloc = '%s%s@%s' % (trust_id, pw, url.netloc)
+                        url = urlparse.SplitResult(url.scheme, netloc,
+                                                   url.path, url.query,
+                                                   url.fragment)
+                        actions[index] = url.geturl()
+        if trust_id is not None and not trust_id_used:
+            prev_trust_ids.add(trust_id)
+        for old_trust_id in prev_trust_ids:
+            keystone_client.delete_trust_id(old_trust_id, auth_plugin)
 
     def delete_actions(self):
-        for action in itertools.chain(self.ok_actions or [],
-                                      self.alarm_actions or [],
-                                      self.insufficient_data_actions or []):
-            self.delete_trust(action)
-
-    def delete_trust(self, action):
         auth_plugin = pecan.request.environ.get('keystone.token_auth')
-        url = netutils.urlsplit(action)
-        if self._is_trust_url(url) and url.password:
-            keystone_client.delete_trust_id(url.username, auth_plugin)
+        for trust_id in self._get_existing_trust_ids():
+            keystone_client.delete_trust_id(trust_id, auth_plugin)
 
 
 Alarm.add_attributes(**{"%s_rule" % ext.name: ext.plugin
@@ -544,7 +553,7 @@ class AlarmController(rest.RestController):
         project_id = pecan.request.headers.get('X-Project-Id')
         on_behalf_of = on_behalf_of or project_id
         severity = scrubbed_data.get('severity')
-        payload = dict(event_id=str(uuid.uuid4()),
+        payload = dict(event_id=uuidutils.generate_uuid(),
                        alarm_id=self._id,
                        type=type,
                        detail=detail,
@@ -563,6 +572,26 @@ class AlarmController(rest.RestController):
         payload['detail'] = scrubbed_data
         _send_notification(type, payload)
 
+    def _record_delete(self, alarm):
+        if not alarm:
+            return
+        type = models.AlarmChange.DELETION
+        detail = {'state': alarm.state}
+        user_id = pecan.request.headers.get('X-User-Id')
+        project_id = pecan.request.headers.get('X-Project-Id')
+        payload = dict(event_id=uuidutils.generate_uuid(),
+                       alarm_id=self._id,
+                       type=type,
+                       detail=detail,
+                       user_id=user_id,
+                       project_id=project_id,
+                       on_behalf_of=project_id,
+                       timestamp=timeutils.utcnow(),
+                       severity=alarm.severity)
+
+        pecan.request.storage.delete_alarm(alarm.alarm_id)
+        _send_notification(type, payload)
+
     @wsme_pecan.wsexpose(Alarm)
     def get(self):
         """Return this alarm."""
@@ -624,7 +653,7 @@ class AlarmController(rest.RestController):
 
         # ensure alarm exists before deleting
         alarm = self._enforce_rbac('delete_alarm')
-        pecan.request.storage.delete_alarm(alarm.alarm_id)
+        self._record_delete(alarm)
         alarm_object = Alarm.from_db_model(alarm)
         alarm_object.delete_actions()
 
@@ -703,7 +732,7 @@ class AlarmsController(rest.RestController):
         user_id = pecan.request.headers.get('X-User-Id')
         project_id = pecan.request.headers.get('X-Project-Id')
         severity = scrubbed_data.get('severity')
-        payload = dict(event_id=str(uuid.uuid4()),
+        payload = dict(event_id=uuidutils.generate_uuid(),
                        alarm_id=alarm_id,
                        type=type,
                        detail=detail,
@@ -734,7 +763,7 @@ class AlarmsController(rest.RestController):
         conn = pecan.request.storage
         now = timeutils.utcnow()
 
-        data.alarm_id = str(uuid.uuid4())
+        data.alarm_id = uuidutils.generate_uuid()
         user_limit, project_limit = rbac.get_limited_to(pecan.request.headers,
                                                         pecan.request.enforcer)
 
@@ -775,7 +804,7 @@ class AlarmsController(rest.RestController):
 
         alarm = conn.create_alarm(alarm_in)
         self._record_creation(conn, change, alarm.alarm_id, now)
-        v2_utils.set_resp_location_hdr("/v2/alarms/" + alarm.alarm_id)
+        v2_utils.set_resp_location_hdr("/alarms/" + alarm.alarm_id)
         return Alarm.from_db_model(alarm)
 
     @wsme_pecan.wsexpose([Alarm], [base.Query], [str], int, str)
