@@ -26,10 +26,10 @@ from futurist import periodics
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import pytz
 import six
 from stevedore import extension
-import uuid
 
 import aodh
 from aodh import coordination
@@ -78,6 +78,14 @@ class Evaluator(object):
             self.storage_conn = storage.get_connection_from_config(self.conf)
         return self.storage_conn
 
+    @property
+    def alarm_change_notifier(self):
+        if not self._alarm_change_notifier:
+            transport = messaging.get_transport(self.conf)
+            self._alarm_change_notifier = messaging.get_notifier(
+                transport, publisher_id="aodh.evaluator")
+        return self._alarm_change_notifier
+
     def _record_change(self, alarm, reason):
         if not self.conf.record_history:
             return
@@ -87,7 +95,7 @@ class Evaluator(object):
         user_id, project_id = self.ks_client.user_id, self.ks_client.project_id
         on_behalf_of = alarm.project_id
         now = timeutils.utcnow()
-        payload = dict(event_id=str(uuid.uuid4()),
+        payload = dict(event_id=uuidutils.generate_uuid(),
                        alarm_id=alarm.alarm_id,
                        type=type,
                        detail=detail,
@@ -100,13 +108,9 @@ class Evaluator(object):
             self._storage_conn.record_alarm_change(payload)
         except aodh.NotImplementedError:
             pass
-        if not self._alarm_change_notifier:
-            transport = messaging.get_transport(self.conf)
-            self._alarm_change_notifier = messaging.get_notifier(
-                transport, publisher_id="aodh.evaluator")
         notification = "alarm.state_transition"
-        self._alarm_change_notifier.info({},
-                                         notification, payload)
+        self.alarm_change_notifier.info({},
+                                        notification, payload)
 
     def _refresh(self, alarm, state, reason, reason_data, always_record=False):
         """Refresh alarm state."""
