@@ -22,6 +22,7 @@ import six.moves.urllib.parse as urlparse
 from aodh.i18n import _LE, _LI
 from aodh import keystone_client
 from aodh import notifier
+from aodh.notifier import trust
 
 LOG = log.getLogger(__name__)
 
@@ -34,7 +35,28 @@ SERVICE_OPTS = [
 
 
 class ZaqarAlarmNotifier(notifier.AlarmNotifier):
-    """Zaqar notifier."""
+    """Zaqar notifier.
+
+    This notifier posts alarm notifications either to a Zaqar subscription or
+    to an existing Zaqar queue with a pre-signed URL.
+
+    To create a new subscription in the service project, use a notification URL
+    of the form:
+
+        zaqar://?topic=example&subscriber=test@example.com&ttl=3600
+
+    Multiple subscribers are allowed. ``ttl` is the time to live of the
+    subscription. The queue will be created automatically, in the service
+    project, with a name based on the topic and the alarm ID.
+
+    To use a pre-signed URL for an existing queue, use a notification URL with
+    the scheme ``zaqar://`` and the pre-signing data from Zaqar in the query
+    string:
+
+        zaqar://?queue_name=example&project_id=foo&
+                 paths=/messages&methods=POST&expires=1970-01-01T00:00Z&
+                 signature=abcdefg
+    """
 
     def __init__(self, conf):
         super(ZaqarAlarmNotifier, self).__init__(conf)
@@ -58,9 +80,9 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
                               " Keystone service catalog."))
         return self._zendpoint
 
-    def get_zaqar_client(self):
+    def _get_client_conf(self):
         conf = self.conf.service_credentials
-        params = {
+        return {
             'auth_opts': {
                 'backend': 'keystone',
                 'options': {
@@ -72,15 +94,17 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
                 }
             }
         }
+
+    def get_zaqar_client(self, conf):
         try:
             from zaqarclient.queues import client as zaqar_client
             return zaqar_client.Client(self._get_endpoint(),
-                                       version=2, conf=params)
+                                       version=2, conf=conf)
         except Exception:
             LOG.error(_LE("Failed to connect to Zaqar service "),
                       exc_info=True)
 
-    def get_presigned_client(self, queue_info):
+    def _get_presigned_client_conf(self, queue_info):
         queue_name = queue_info.get('queue_name', [''])[0]
         if not queue_name:
             return None, None
@@ -90,7 +114,7 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
         paths = queue_info.get('paths', [''])[0].split(',')
         methods = queue_info.get('methods', [''])[0].split(',')
         project_id = queue_info.get('project_id', [''])[0]
-        params = {
+        conf = {
             'auth_opts': {
                 'backend': 'signed-url',
                 'options': {
@@ -102,14 +126,7 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
                 }
             }
         }
-        try:
-            from zaqarclient.queues import client as zaqar_client
-            return (zaqar_client.Client(self._get_endpoint(),
-                                        version=2, conf=params),
-                    queue_name)
-        except Exception:
-            LOG.error(_LE("Failed to connect to Zaqar service "),
-                      exc_info=True)
+        return conf, queue_name
 
     def notify(self, action, alarm_id, alarm_name, severity, previous,
                current, reason, reason_data, headers=None):
@@ -128,23 +145,25 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
                 'current': current, 'reason': reason,
                 'reason_data': reason_data}
         message = dict(body=body)
-        self.notify_zaqar(action, message)
+        self.notify_zaqar(action, message, headers)
 
     @property
     def client(self):
         if self._zclient is None:
-            self._zclient = self.get_zaqar_client()
+            self._zclient = self.get_zaqar_client(self._get_client_conf())
         return self._zclient
 
-    def notify_zaqar(self, action, message):
+    def notify_zaqar(self, action, message, headers=None):
         queue_info = urlparse.parse_qs(action.query)
         try:
             # NOTE(flwang): Try to get build a pre-signed client if user has
             # provide enough information about that. Otherwise, go to build
             # a client with service account and queue name for this alarm.
-            zaqar_client, queue_name = self.get_presigned_client(queue_info)
+            conf, queue_name = self._get_presigned_client_conf(queue_info)
+            if conf is not None:
+                zaqar_client = self.get_zaqar_client(conf)
 
-            if not zaqar_client or not queue_name:
+            if conf is None or queue_name is None or zaqar_client is None:
                 zaqar_client = self.client
                 # queue_name is a combination of <alarm-id>-<topic>
                 queue_name = "%s-%s" % (message['body']['alarm_id'],
@@ -163,8 +182,46 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
             # post the message to the queue
             queue.post(message)
         except IndexError:
-            LOG.error(_LE("Required topic query option missing in action %s")
-                      % action)
+            LOG.error(_LE("Required query option missing in action %s"),
+                      action)
+        except Exception:
+            LOG.error(_LE("Unknown error occurred; Failed to post message to"
+                          " Zaqar queue"),
+                      exc_info=True)
+
+
+class TrustZaqarAlarmNotifier(trust.TrustAlarmNotifierMixin,
+                              ZaqarAlarmNotifier):
+    """Zaqar notifier using a Keystone trust to post to user-defined queues.
+
+    The URL must be in the form ``trust+zaqar://trust_id@?queue_name=example``.
+    """
+
+    def _get_client_conf(self, auth_token):
+        return {
+            'auth_opts': {
+                'backend': 'keystone',
+                'options': {
+                    'os_auth_token': auth_token,
+                }
+            }
+        }
+
+    def notify_zaqar(self, action, message, headers):
+        queue_info = urlparse.parse_qs(action.query)
+        try:
+            queue_name = queue_info.get('queue_name')[-1]
+        except IndexError:
+            LOG.error(_LE("Required 'queue_name' query option missing in"
+                          " action %s"),
+                      action)
+            return
+
+        try:
+            conf = self._get_client_conf(headers['X-Auth-Token'])
+            client = self.get_zaqar_client(conf)
+            queue = client.queue(queue_name)
+            queue.post(message)
         except Exception:
             LOG.error(_LE("Unknown error occurred; Failed to post message to"
                           " Zaqar queue"),
