@@ -17,14 +17,13 @@
 """
 import functools
 import os.path
+import unittest
 
-import eventlet
 import oslo_messaging.conffixture
 from oslo_utils import timeutils
 from oslotest import base
 from oslotest import mockpatch
 import six
-from testtools import testcase
 import webtest
 
 import aodh
@@ -39,11 +38,6 @@ class BaseTestCase(base.BaseTestCase):
             exchange = 'aodh'
         conf.set_override("control_exchange", exchange)
 
-        # NOTE(sileht): oslo.messaging fake driver uses time.sleep
-        # for task switch, so we need to monkey_patch it
-        # and also ensure the correct exchange have been set
-        eventlet.monkey_patch(time=True, thread=True)
-
         # NOTE(sileht): Ensure a new oslo.messaging driver is loaded
         # between each tests
         self.transport = messaging.get_transport(conf, "fake://", cache=False)
@@ -96,10 +90,10 @@ def _skip_decorator(func):
         try:
             return func(*args, **kwargs)
         except aodh.NotImplementedError as e:
-            raise testcase.TestSkipped(six.text_type(e))
+            raise unittest.SkipTest(six.text_type(e))
         except webtest.app.AppError as e:
             if 'not implemented' in six.text_type(e):
-                raise testcase.TestSkipped(six.text_type(e))
+                raise unittest.SkipTest(six.text_type(e))
             raise
     return skip_if_not_implemented
 
