@@ -16,26 +16,41 @@
 """Fixtures used during Gabbi-based test runs."""
 
 import os
-import uuid
 
 from gabbi import fixture
 import mock
 from oslo_config import cfg
 from oslo_config import fixture as fixture_config
 from oslo_policy import opts
+from oslo_utils import uuidutils
 from six.moves.urllib import parse as urlparse
 import sqlalchemy_utils
 
+from aodh.api import app
 from aodh import service
 from aodh import storage
 
 
+# NOTE(chdent): Hack to restore semblance of global configuration to
+# pass to the WSGI app used per test suite. LOAD_APP_KWARGS are the olso
+# configuration, and the pecan application configuration of
+# which the critical part is a reference to the current indexer.
+LOAD_APP_KWARGS = None
+
+
+def setup_app():
+    global LOAD_APP_KWARGS
+    return app.load_app(**LOAD_APP_KWARGS)
+
+
 class ConfigFixture(fixture.GabbiFixture):
     """Establish the relevant configuration for a test run."""
 
     def start_fixture(self):
         """Set up config."""
 
+        global LOAD_APP_KWARGS
+
         self.conf = None
         self.conn = None
 
@@ -66,19 +81,12 @@ class ConfigFixture(fixture.GabbiFixture):
                               'aodh/tests/open-policy.json'),
                           group='oslo_policy',
                           enforce_type=True)
-        conf.set_override(
-            'paste_config',
-            os.path.abspath('aodh/tests/functional/gabbi/gabbi_paste.ini'),
-            group='api',
-        )
-
-        conf.set_override('pecan_debug', True, group='api',
-                          enforce_type=True)
+        conf.set_override('auth_mode', None, group='api')
 
         parsed_url = urlparse.urlparse(db_url)
         if parsed_url.scheme != 'sqlite':
             parsed_url = list(parsed_url)
-            parsed_url[2] += '-%s' % str(uuid.uuid4()).replace('-', '')
+            parsed_url[2] += '-%s' % uuidutils.generate_uuid(dashed=False)
             db_url = urlparse.urlunparse(parsed_url)
 
         conf.set_override('connection', db_url, group='database',
@@ -91,6 +99,10 @@ class ConfigFixture(fixture.GabbiFixture):
         self.conn = storage.get_connection_from_config(self.conf)
         self.conn.upgrade()
 
+        LOAD_APP_KWARGS = {
+            'conf': conf,
+        }
+
     def stop_fixture(self):
         """Reset the config and remove data."""
         if self.conn:
